#!/usr/bin/env python3

import gi
gi.require_version('Gtk', '3.0')
gi.require_version('Clutter', '1.0')
gi.require_version('GtkClutter', '1.0')
from gi.repository import Gtk, Gio, GLib

from comipoli_window import ComipoliWindow
from comipoli_shortcuts import ComipoliShortcuts
from comipoli_preference import ComipoliPreference

VERSION="0.0.beta14"

class ComipoliApplication(Gtk.Application):
    #__gtype_name__ = "ComipoliApplication"
    def __init__(self):
        GLib.set_prgname("Comipoli")
        Gtk.Application.__init__(
            self,
            application_id="org.sasakima.comipoli",
            flags=Gio.ApplicationFlags.HANDLES_OPEN
        )

    def do_startup(self):
        Gtk.Application.do_startup(self)
        # Dark Theme
        settings = Gtk.Settings.get_default()
        settings.props.gtk_application_prefer_dark_theme = True
        # ApplicationMenu
        menu = Gio.Menu()
        menu.append("_Preference", "app.preference_action")
        menu.append("_Keyboard Shortcuts", "app.shortcut_action")
        menu.append("_About", "app.about_action")
        menu.append("_Quit", "app.quit_action")
        self.set_app_menu(menu)
        # Accel
        self.set_accels_for_action("app.shortcut_action", ["<Control>F1", "question"])
        self.set_accels_for_action("app.quit_action", ["<Control>Q"])
        # Action
        preference_action = Gio.SimpleAction(name="preference_action")
        shortcut_action   = Gio.SimpleAction(name="shortcut_action")
        about_action      = Gio.SimpleAction(name="about_action")
        quit_action       = Gio.SimpleAction(name="quit_action")
        self.add_action(preference_action)
        self.add_action(shortcut_action)
        self.add_action(about_action)
        self.add_action(quit_action)
        # connect
        preference_action.connect("activate", self.on_preference_action)
        shortcut_action.connect("activate", self.on_shortcut_action)
        about_action.connect("activate", self.on_about_action)
        quit_action.connect("activate", self.on_quit_action)
        #
        self.window = ComipoliWindow(self)
        self.shortcuts = ComipoliShortcuts()
        self.window.set_help_overlay(self.shortcuts)

    def on_preference_action(self, action, parameter):
        dlg = ComipoliPreference(self.window)
        dlg.run()
        dlg.destroy()

    def on_shortcut_action(self, action, parameter):
        self.shortcuts.show_all()

    def on_about_action(self, action, parameter):
        d = Gtk.AboutDialog(
            comments="Comic book Archive (CBZ/CBR/CB7) Viewer",
            copyright="Copyright(C) 2016-2017 sasakima-nao",
            #license="GPL",
            license_type=Gtk.License.GPL_2_0,
            logo=self.window.get_icon(),
            program_name="Comipoli",
            version=VERSION,
            website="http://palepoli.skr.jp/"
        )
        d.set_transient_for(self.get_active_window())
        d.run()
        d.destroy()

    def on_quit_action(self, action, parameter):
        self.quit()

    def do_open(self, files, n_file, hint):
        uri = files[0].get_uri()
        self.window.set_uri(uri)
        self.window.present()

    def do_activate(self):
        self.window.present()

