#!/usr/bin/env python3

from gi.repository import GLib, Gio, GdkPixbuf
import os, zipfile

class ComipoliArchive:
    def __init__(self):
        self.status = 0
        self.namelist = []

    def new_cba(self, uri, is_unrar, is_7za):
        self.arc = Gio.file_new_for_uri(uri)
        self.path = self.arc.get_path()
        #ext = os.path.splitext(self.path)[1].lower()
        #if ext == ".cbz":
        if GLib.Regex.match_simple("\.(cbz|zip)$", self.path, GLib.RegexCompileFlags.CASELESS, 0):
            self.status = 0
            self.namelist.clear()
            with zipfile.ZipFile(self.path) as o:
                l = o.namelist()
                l.sort()
                for name in l:
                    if GLib.Regex.match_simple("\.(jpe?g|png|gif)$", name, GLib.RegexCompileFlags.CASELESS, 0):
                        self.namelist.append(name)
        #elif is_unrar and ext == ".cbr":
        elif is_unrar and GLib.Regex.match_simple("\.(cbr|rar)$", self.path, GLib.RegexCompileFlags.CASELESS, 0):
            self.status = 1
            self.namelist.clear()
            result, output, error, status = GLib.spawn_command_line_sync('unrar vt -p- -- "{0}"'.format(self.path))
            lines = output.decode("utf-8").split("\n")
            for line in lines:
                s = line.lstrip()
                if s.startswith('Name: '):
                    name = s[6:]
                    if GLib.Regex.match_simple("\.(jpe?g|png|gif)$", name, GLib.RegexCompileFlags.CASELESS, 0):
                        self.namelist.append(name)
        #elif is_7za and ext == ".cb7":
        elif is_7za and GLib.Regex.match_simple("\.(cb7|7z)$", self.path, GLib.RegexCompileFlags.CASELESS, 0):
            self.status = 2
            self.namelist.clear()
            result, output, error, status = GLib.spawn_command_line_sync('7za l -slt "{0}"'.format(self.path))
            lines = output.decode("utf-8").split("\n")
            for line in lines:
                if line.startswith('Path'):
                    name = line[7:]
                    if GLib.Regex.match_simple("\.(jpe?g|png|gif)$", name, GLib.RegexCompileFlags.CASELESS, 0):
                        self.namelist.append(name)
        else:
            return False
        return True

    def __getitem__(self, num):
        stream = None
        if (self.status == 0):
            with zipfile.ZipFile(self.path) as o:
                data = o.read(self.namelist[num])
                stream = Gio.MemoryInputStream.new_from_data(data)
        elif (self.status == 1):
            sp = Gio.Subprocess.new(["unrar", "p", "-inul", "-@", "--", self.path, self.namelist[num]], Gio.SubprocessFlags.STDOUT_PIPE)
            stream = sp.get_stdout_pipe()
        elif (self.status == 2):
            sp = Gio.Subprocess.new(["7za", "x", "-so", self.path, self.namelist[num]], Gio.SubprocessFlags.STDOUT_PIPE)
            stream = sp.get_stdout_pipe()
        try:
            p = GdkPixbuf.Pixbuf.new_from_stream(stream)
        except Exception as e:
            print(self.namelist[num])
            print(e)
        stream.close()
        return p

    def __len__(self):
        return len(self.namelist)
